from typing import Protocol, runtime_checkable
import json
from importlib import import_module
from datetime import datetime


@runtime_checkable
class Saveable(Protocol):
    """
    this protocol should be implemented in every class
    that should be save/load-able from a dict.
    keep in mind, that only native python dtypes work wiht json
    """
    @property
    def qualifiedName(self) -> tuple:
        ...

    def toDict(self) -> dict:
        pass

    @classmethod
    def fromDict(cls: object, loadDict: dict) -> object:
        pass


class ModelIO(object):
    @classmethod
    def load(self, filename: str) -> object:
        with open(f'{filename}.json', 'r') as f:
            loadDict = json.load(f)

        moduleName, className = loadDict.pop('qualifiedName')  # assuming qualifiedName exists in the dict
        loadDict.pop('datetime')

        Module = import_module(moduleName)  # dynamically import module
        Class = getattr(Module, className)  # get class from imported module

        try:
            return Class.fromDict(loadDict)
        except AttributeError:
            raise TypeError(f'{className} is not Saveable.')

    @classmethod
    def save(self, object: object, filename: str) -> None:
        if not isinstance(object, Saveable):
            raise TypeError(f'{object.__class__.__name__} is not Saveable.')
        saveDict = object.toDict()
        saveDict['datetime'] = datetime.now().isoformat()
        saveDict['qualifiedName'] = object.qualifiedName


        with open(f'{filename}.json', 'w') as f:
            json.dump(saveDict, f)
